// FileServices.cpp: implementation of the CFileServices class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PropCom.h"
#include "FileServices.h"
#include <tlhelp32.h>
#include <intshcut.h>
#include <shlwapi.h>
// link to shlwapi.lib in project settings

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

BOOL CALLBACK TerminateAppEnum(HWND hwnd, LPARAM lParam)
{
	DWORD dwID;

	GetWindowThreadProcessId(hwnd, &dwID);

	if(dwID == (DWORD)lParam)
	{
		PostMessage(hwnd, WM_CLOSE, 0, 0);
	}
	return TRUE;
}

CFileServices::CFileServices()
{
	if(InitializeDirPaths_FS() == FALSE)
	{
		CString strErrorMessage;
		strErrorMessage += "This program";
		strErrorMessage += " has experienced a runtime error"
			" and must be shut down";
		MessageBox(NULL, strErrorMessage, "Runtime Error", MB_OK);

		ExitProcess(0);
	}
}

CFileServices::~CFileServices()
{

}

BOOL CFileServices::InitializeDirPaths_FS()
{
	BOOL bDirIniResult = TRUE;

	HKEY hKey;

	DWORD lpcbData;	

	GetWindowsDirectory(szWindowsDir, MAX_PATH);

	lstrcpyn(szMainDrive, (LPCTSTR)szWindowsDir, 3);

	RegOpenKeyEx (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
		0, KEY_QUERY_VALUE, &hKey);

	RegQueryValueEx (hKey, "ProgramFilesDir", NULL,
		NULL, NULL, &lpcbData);

	RegQueryValueEx (hKey, "ProgramFilesDir", NULL,
		NULL, (LPBYTE)(LPTSTR)szProgramFilesDir, &lpcbData);

	RegCloseKey (hKey);	
	
	//  This is only a few of the available folders
	// and the following path functions are just a few
	// of those which are listed in shlwapi.h
	SHGetSpecialFolderPath(NULL, szProgramsDir, CSIDL_PROGRAMS, 0);	
	SHGetSpecialFolderPath(NULL, szDesktopDir, CSIDL_DESKTOPDIRECTORY, 0);	
	SHGetSpecialFolderPath(NULL, szStartupDir, CSIDL_STARTUP, 0);	
	SHGetSpecialFolderPath(NULL, szStartMenuDir, CSIDL_STARTMENU, 0);	
	SHGetSpecialFolderPath(NULL, szMyDocumentsDir, CSIDL_PERSONAL, 0);	
	SHGetSpecialFolderPath(NULL, szFavoritesDir, CSIDL_FAVORITES, 0);

	if(strcmp(szWindowsDir, _T("")) == 0 || 
		strcmp(szMainDrive, _T("")) == 0 ||
		strcmp(szProgramFilesDir, _T("")) == 0 ||
		strcmp(szProgramsDir, _T("")) == 0 ||
		strcmp(szDesktopDir, _T("")) == 0 ||
		strcmp(szStartupDir, _T("")) == 0 ||
		strcmp(szStartMenuDir, _T("")) == 0 ||
		strcmp(szMyDocumentsDir, _T("")) == 0 ||
		strcmp(szFavoritesDir, _T("")) == 0)
	{
		bDirIniResult = FALSE;
	}

	return bDirIniResult;
}

BOOL CFileServices::CopyFile_FS(LPTSTR lpExistingFileName, LPTSTR lpNewFileName, BOOL bFailIfExists)
{
	return CopyFile((LPCTSTR)lpExistingFileName, (LPCTSTR)lpNewFileName, bFailIfExists);
}

BOOL CFileServices::MoveFile_FS(LPTSTR lpExistingFileName, LPTSTR lpNewFileName)
{
	return MoveFile((LPCTSTR)lpExistingFileName, (LPCTSTR)lpNewFileName);
}

BOOL CFileServices::DeleteFile_FS(LPTSTR lpFileName)
{
	return DeleteFile((LPCTSTR)lpFileName);
}

BOOL CFileServices::CreateDirectory_FS(LPTSTR lpszDirPath)
{
	return CreateDirectory((LPCTSTR)lpszDirPath, NULL);
}

void CFileServices::DeleteDirectory_FS(LPTSTR szDirectoryPath)
{
	SHFILEOPSTRUCT shfo;

	ZeroMemory(&shfo, sizeof(SHFILEOPSTRUCT));

	shfo.hwnd = NULL;
	shfo.wFunc = FO_DELETE;
	shfo.lpszProgressTitle = "Deleting Files";
	shfo.fFlags = FOF_SIMPLEPROGRESS | FOF_NOCONFIRMATION | FOF_NOERRORUI;
	shfo.pFrom = szDirectoryPath;

	SHFileOperation(&shfo);
}

BOOL CFileServices::RemoveDirectory_FS(LPTSTR lpszDirPath)
{
	return RemoveDirectory((LPCTSTR)lpszDirPath);
}

LPTSTR CFileServices::PathAddBackslash_FS(LPTSTR lpszPath)
{
	return PathAddBackslash(lpszPath);
}

BOOL CFileServices::PathAddExtension_FS(LPTSTR pszPath, LPCTSTR pszExtension)
{
	return PathAddExtension(pszPath, pszExtension);
}

BOOL CFileServices::PathAppend_FS(LPTSTR pszPath, LPCTSTR pszMore)
{
	return PathAppend(pszPath, pszMore);
}

LPTSTR CFileServices::PathRemoveBackslash_FS(LPTSTR lpszPath)
{
	return PathRemoveBackslash(lpszPath);
}

BOOL CFileServices::PathFileExists_FS(LPTSTR pszPath)
{
	return PathFileExists((LPCTSTR)pszPath);
}

LPTSTR CFileServices::PathGetArgs_FS(LPTSTR pszPath)
{
	return PathGetArgs((LPCTSTR) pszPath);
}

BOOL CFileServices::PathIsDirectory_FS(LPTSTR pszPath)
{
	return PathIsDirectory((LPCTSTR)pszPath);
}

BOOL CFileServices::PathIsFileSpec_FS(LPTSTR pszPath)
{
	return PathIsFileSpec((LPCTSTR)pszPath);
}

void CFileServices::PathRemoveArgs_FS(LPTSTR pszPath)
{
	PathRemoveArgs(pszPath);
}

void CFileServices::PathRemoveExtension_FS(LPTSTR pszPath)
{
	PathRemoveExtension(pszPath);
}

BOOL CFileServices::PathRemoveFileSpec_FS(LPTSTR pszPath)
{
	return PathRemoveFileSpec(pszPath);
}

BOOL CFileServices::PathRenameExtension_FS(LPTSTR pszPath, LPCTSTR pszExt)
{
	return PathRenameExtension(pszPath, pszExt);
}

BOOL CFileServices::PathStripToRoot_FS(LPTSTR szRoot)
{
	return PathStripToRoot(szRoot);
}

BOOL CFileServices::CreateShellLink_FS(LPTSTR lpszLinkFile, LPTSTR lpszLinkPath, LPTSTR lpszIconFile, UINT idIcon, LPCTSTR pszCmdLineArgs)
{
	IShellLink *pLink = NULL;
	IPersistFile *pFile = NULL;
	HRESULT hResult = NOERROR;
	BOOL bResult = FALSE;
	WCHAR szLinkFile[MAX_PATH] = L"";

	if (lpszLinkFile == NULL)
	{
		SetLastError (ERROR_INVALID_PARAMETER);
		return FALSE;
	}

	#ifdef UNICODE
		lstrcpy (szLinkFile, lpszLinkFile);
	#else
		MultiByteToWideChar (CP_ACP, 0, lpszLinkFile, -1, szLinkFile, MAX_PATH);
	#endif
        
	hResult = CoCreateInstance (CLSID_ShellLink, NULL, 
		CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*) &pLink);

	if (FAILED(hResult))
		goto done;

	hResult = pLink->QueryInterface (IID_IShellLink, (LPVOID*) &pLink);
	if (FAILED(hResult))
		goto done;

	hResult = pLink->QueryInterface (IID_IPersistFile, (LPVOID*) &pFile);
	if (FAILED(hResult))
		goto done;

	hResult = pLink->SetPath (lpszLinkPath);
	if (FAILED(hResult))
		goto done;

	if(pszCmdLineArgs != _T(""))
	{
		pLink->SetArguments (pszCmdLineArgs);
	}

	if (lpszIconFile)
	{
		hResult = pLink->SetIconLocation (lpszIconFile, idIcon);
	}

	hResult = pFile->Save (szLinkFile, TRUE);
	if (FAILED(hResult))
		goto done;

	bResult = TRUE;

	done:

	if (pFile)
	{
		pFile->Release();
		pFile = NULL;
	}

	if (pLink)
	{
		pLink->Release();
		pLink = NULL;
	}

	SetLastError ((DWORD)hResult);
	return bResult;
}

BOOL CFileServices::CreateInternetShortcut_FS(LPTSTR lpszLinkFile, LPCTSTR lpszURL, LPTSTR lpszIconFile, UINT idIcon)
{
	IUniformResourceLocator *pURL = NULL;
	IShellLink *pLink = NULL;
	IPersistFile *pFile = NULL;
	HRESULT hResult = NOERROR;
	BOOL bResult = FALSE;
	WCHAR szLinkFile[MAX_PATH] = L"";

	if (lpszURL == NULL || lpszLinkFile == NULL)
	{
		SetLastError (ERROR_INVALID_PARAMETER);
		return FALSE;
	}

	#ifdef UNICODE
		lstrcpy (szLinkFile, lpszLinkFile);
	#else
		MultiByteToWideChar (CP_ACP, 0, lpszLinkFile, -1, szLinkFile, MAX_PATH);
	#endif
        
	hResult = CoCreateInstance (CLSID_InternetShortcut, NULL,
		CLSCTX_INPROC_SERVER, IID_IUniformResourceLocator, (LPVOID*)&pURL);

	if (FAILED(hResult))
		goto done;

	hResult = pURL->QueryInterface (IID_IShellLink, (LPVOID*) &pLink);
	if (FAILED(hResult))
		goto done;

	hResult = pURL->QueryInterface (IID_IPersistFile, (LPVOID*) &pFile);
	if (FAILED(hResult))
		goto done;

	hResult = pURL->SetURL (lpszURL, IURL_SETURL_FL_GUESS_PROTOCOL);
	if (FAILED(hResult))
		goto done;

	if (lpszIconFile)
	{
		hResult = pLink->SetIconLocation (lpszIconFile, idIcon);
	}

	hResult = pFile->Save (szLinkFile, TRUE);
	if (FAILED(hResult))
		goto done;

	bResult = TRUE;

	done:

	if (pFile)
	{
		pFile->Release();
		pFile = NULL;
	}

	if (pLink)
	{
		pLink->Release();
		pLink = NULL;
	}

	if (pURL)
	{
		pURL->Release();
		pURL = NULL;
	}

	SetLastError ((DWORD)hResult);
	return bResult;
}

void CFileServices::WriteIconResourceFromExe_FS(UINT idIcon, LPTSTR szPath)
{
	LPICONRESOURCE lpIR;
	LPMEMICONDIR lpIcon;
	LPICONIMAGE lpImage;

	HRSRC hRsrc;
	HGLOBAL hGlobal;
	HANDLE hFile;

	UINT i;
	UINT nIndex;
	UINT nNumEntries;
    
	DWORD dwBytesWrittenOne;
	DWORD dwSize;
	DWORD dwBytesWrittenTwo;

	WORD Output;

	hRsrc = FindResource(NULL, (LPTSTR)idIcon, RT_GROUP_ICON);
	hGlobal = LoadResource(NULL, hRsrc);
	lpIcon = (LPMEMICONDIR)LockResource(hGlobal);

	lpIR = (LPICONRESOURCE)malloc(sizeof(ICONRESOURCE) +
		((lpIcon->idCount-1) * sizeof(ICONIMAGE)));

	lpIR->nNumImages = lpIcon->idCount;

	for (i = 0; i < lpIR->nNumImages; i++)
	{
		hRsrc = FindResource(NULL, MAKEINTRESOURCE(lpIcon->idEntries[i].nID), RT_ICON);
		hGlobal = LoadResource(NULL, hRsrc);

		lpIR->IconImages[i].dwNumBytes = SizeofResource (NULL, hRsrc);
		lpIR->IconImages[i].lpBits = (LPBYTE)malloc( lpIR->IconImages[i].dwNumBytes);
		memcpy (lpIR->IconImages[i].lpBits, LockResource (hGlobal),
		lpIR->IconImages[i].dwNumBytes);

		lpImage = &(lpIR->IconImages[i]);
		lpImage->lpbi = (LPBITMAPINFO)lpImage->lpBits;
		lpImage->Width = lpImage->lpbi->bmiHeader.biWidth;
		lpImage->Height = (lpImage->lpbi->bmiHeader.biHeight) / 2;
		lpImage->Colors = lpImage->lpbi->bmiHeader.biPlanes *
			lpImage->lpbi->bmiHeader.biBitCount;
	}

	hFile = CreateFile( szPath, GENERIC_WRITE, 0, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

	nNumEntries = lpIR->nNumImages;

	Output = 0;
	WriteFile( hFile, &Output, sizeof(WORD), &dwBytesWrittenTwo, NULL);

	Output = 1;
	WriteFile( hFile, &Output, sizeof(WORD), &dwBytesWrittenTwo, NULL);

	Output = (WORD)nNumEntries;

	WriteFile( hFile, &Output, sizeof(WORD), &dwBytesWrittenTwo, NULL);

	for( i=0; i<lpIR->nNumImages; i++ )
	{
		ICONDIRENTRY ide;

		ide.bWidth = lpIR->IconImages[i].Width;
		ide.bHeight = lpIR->IconImages[i].Height;
		ide.bReserved = 0;
		ide.wPlanes = lpIR->IconImages[i].lpbi->bmiHeader.biPlanes;
		ide.wBitCount = lpIR->IconImages[i].lpbi->bmiHeader.biBitCount;

		if( (ide.wPlanes * ide.wBitCount) >= 8)
			ide.bColorCount = 0;
		else
			ide.bColorCount = 1 << (ide.wPlanes * ide.wBitCount);

		ide.dwBytesInRes = lpIR->IconImages[i].dwNumBytes;
		dwSize = 3 * sizeof(WORD);
		dwSize += lpIR->nNumImages * sizeof(ICONDIRENTRY);

		nIndex = i;

		for(i=0; i<nIndex; i++)
			dwSize += lpIR->IconImages[i].dwNumBytes;

		ide.dwImageOffset = dwSize;

		WriteFile( hFile, &ide, sizeof(ICONDIRENTRY), &dwBytesWrittenOne, NULL);
	}

	for( i=0; i<lpIR->nNumImages; i++ )
	{
		DWORD dwTemp = lpIR->IconImages[i].lpbi->bmiHeader.biSizeImage;

		lpIR->IconImages[i].lpbi->bmiHeader.biSizeImage = 0;

		WriteFile( hFile, lpIR->IconImages[i].lpBits, lpIR->IconImages[i].dwNumBytes,
			&dwBytesWrittenOne, NULL);

		lpIR->IconImages[i].lpbi->bmiHeader.biSizeImage = dwTemp;
	}

	CloseHandle( hFile );
	UnlockResource(hGlobal);
	FreeResource(hGlobal);

	for (i = 0; i < lpIR->nNumImages; i++)
	{
		free (lpIR->IconImages[i].lpBits);
	}

	free (lpIR);
}

void CFileServices::WriteBitmapResourceFromExe_FS(UINT idBitmap, LPTSTR szPath)
{
	HANDLE hFile;
	BITMAPFILEHEADER bmfh;
	HRSRC hRsrc;
	HGLOBAL hGlobal;
	PBITMAPINFO pbmi;
    DWORD dwBytesWritten;

	hRsrc = FindResource(NULL, (LPSTR)idBitmap, RT_BITMAP);
	hGlobal = LoadResource(NULL, hRsrc);
	pbmi = (PBITMAPINFO)LockResource(hGlobal);

	hFile = CreateFile(szPath, GENERIC_READ | GENERIC_WRITE,
		(DWORD)0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE,
		(HANDLE)NULL);

	bmfh.bfOffBits = 1078;
	bmfh.bfType = 0x4d42;
	bmfh.bfSize = pbmi->bmiHeader.biSizeImage + bmfh.bfOffBits;
	bmfh.bfReserved1 = 0;
    bmfh.bfReserved2 = 0;

    WriteFile(hFile, &bmfh, sizeof(BITMAPFILEHEADER), (LPDWORD)&dwBytesWritten, NULL);
	WriteFile(hFile, pbmi, pbmi->bmiHeader.biSizeImage + bmfh.bfOffBits - sizeof(BITMAPFILEHEADER), (LPDWORD)&dwBytesWritten, NULL);
	
    CloseHandle(hFile);

    UnlockResource(hGlobal);
    FreeResource(hGlobal);
}

void CFileServices::WriteCustomResourceFromExe_FS(UINT idCustom,
	LPCTSTR lpszCustomType, LPTSTR szPath, BOOL bBinary)
{
	HMODULE hModule;
	HRSRC hRsrc;
	HGLOBAL hGlobal;
	BYTE* pBytes;
	DWORD dwLength;
	FILE* fOutput;

	hModule = GetModuleHandle(NULL);
	hRsrc = FindResource(hModule, MAKEINTRESOURCE(idCustom), lpszCustomType);
	hGlobal = LoadResource(hModule, hRsrc);
	pBytes = (BYTE*) LockResource(hGlobal);
	dwLength = SizeofResource(hModule, hRsrc);

	if(bBinary == TRUE)
	{
		fOutput = fopen(szPath, "wb");
	}
	else
	{
		fOutput = fopen(szPath, "wt");
	}

	fwrite(pBytes, 1, dwLength, fOutput);
	fclose(fOutput);
	UnlockResource(hGlobal);
	FreeResource(hGlobal);
}

BOOL CFileServices::CloseAppNow(LPTSTR szRunningApp)
{
	// Enumerate and terminate all processes named szRunningApp. This code calls 
	// the TerminateProcess API, which is usually not recommended. We must  
	// use this here, however, because Explorer's auto-restart will cause       // logoff problems.

	HINSTANCE hInstLib;
	HANDLE hSnapShot;
	PROCESSENTRY32 procentry;
	BOOL bFlag;
	BOOL bAppIsRunning = FALSE;

	// ToolHelp function pointers.
	HANDLE (WINAPI *lpfCreateToolhelp32Snapshot)(DWORD,DWORD);
	BOOL (WINAPI *lpfProcess32First)(HANDLE,LPPROCESSENTRY32);
	BOOL (WINAPI *lpfProcess32Next)(HANDLE,LPPROCESSENTRY32);

	hInstLib = LoadLibraryA("Kernel32.DLL");
	
	if(hInstLib == NULL)
	{
		return FALSE;
	}

	// Get procedure addresses.
	// We are linking to these functions of Kernel32
	// explicitly, because otherwise a module using                                                                    
	// this code would fail to load under Windows NT,
	// which does not have the Toolhelp32
	// functions in the Kernel 32.
	lpfCreateToolhelp32Snapshot=
		(HANDLE(WINAPI *)(DWORD,DWORD))
		GetProcAddress(hInstLib,
		"CreateToolhelp32Snapshot");
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
	lpfProcess32First=
		(BOOL(WINAPI *)(HANDLE,LPPROCESSENTRY32))
		GetProcAddress(hInstLib, "Process32First");

	lpfProcess32Next=
		(BOOL(WINAPI *)(HANDLE,LPPROCESSENTRY32))
		GetProcAddress(hInstLib, "Process32Next");

	if(lpfProcess32Next == NULL ||
		lpfProcess32First == NULL ||
		lpfCreateToolhelp32Snapshot == NULL)
	{
		FreeLibrary(hInstLib);
		return FALSE;
	}

	// Get a handle to a ToolHelp snapshot of the systems
	// processes.
	hSnapShot = lpfCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	if(hSnapShot == INVALID_HANDLE_VALUE)
	{
		FreeLibrary(hInstLib);
		return FALSE;
	}

	// Get the first process's information.
	procentry.dwSize = sizeof(PROCESSENTRY32);
	bFlag = lpfProcess32First(hSnapShot, &procentry);

	// While there are processes, keep looping.
	while(bFlag)
	{
		{
			// We need to see if the app were looking for is executing
			HANDLE hProcess;
			INT nPos;

			nPos = lstrlen(procentry.szExeFile);

			if(nPos)
			{
				while (procentry.szExeFile[--nPos] != '\\');
				if(!lstrcmpi(szRunningApp,&(procentry.szExeFile[nPos+1])))
				{
					// Terminate the process.
					hProcess = OpenProcess(PROCESS_ALL_ACCESS,FALSE,
						procentry.th32ProcessID);

					EnumWindows((WNDENUMPROC)TerminateAppEnum,
						(LPARAM)procentry.th32ProcessID);

					if(WaitForSingleObject(hProcess, 600) != WAIT_OBJECT_0)
					{
						TerminateProcess(hProcess, 1);
					}

					CloseHandle(hProcess);

					bAppIsRunning = TRUE;
				}
			}
		}

		procentry.dwSize = sizeof(PROCESSENTRY32);
		bFlag = lpfProcess32Next(hSnapShot, &procentry);
	}

	CloseHandle(hSnapShot);

	// Free the library.
	FreeLibrary(hInstLib);

	if(bAppIsRunning == TRUE)
	{
		return TRUE;
	}
	else
	{
		return 0;
	}
}